from anyrun.iterators.threat_intelligence.yara_lookup import StixYaraIterator, JsonYaraIterator
from anyrun.connectors import YaraLookupConnector


class YaraIterator:
    """ Iterator Factory. Creates a concrete iterator instance according to the method called """
    @staticmethod
    def stix(
            connector: YaraLookupConnector,
            yara_rule: str,
            chunk_size: int = 1
    ) -> StixYaraIterator:
        """
        Iterates through the yara search matches. Returns matches in **json** format

        :param connector: Connector instance
        :param yara_rule: Valid YARA rule
        :param chunk_size: The number of feed objects to be retrieved each iteration.
            If greater than one, returns the list of objects
        :return: StixYaraIterator instance
        """
        return StixYaraIterator(
            connector=connector,
            chunk_size=chunk_size,
            yara_rule=yara_rule
        )

    @staticmethod
    def json(
            connector: YaraLookupConnector,
            yara_rule: str,
            chunk_size: int = 1
    ) -> JsonYaraIterator:
        """
        Iterates through the yara search matches. Returns matches in **json** format

        :param connector: Connector instance
        :param yara_rule: Valid YARA rule
        :param chunk_size: The number of feed objects to be retrieved each iteration.
            If greater than one, returns the list of objects
        """
        return JsonYaraIterator(
            connector=connector,
            chunk_size=chunk_size,
            yara_rule=yara_rule
        )



from anyrun.connectors.threat_intelligence.feeds_connector import FeedsConnector
from anyrun.connectors.threat_intelligence.lookup_connector import LookupConnector
from anyrun.connectors.threat_intelligence.yara_lookup_connector import YaraLookupConnector

from anyrun.connectors.sandbox.sandbox_connector import SandBoxConnector



from typing import Optional


class RunTimeException(Exception):
    def __init__(self, reason: dict) -> None:
        self._reason = reason

    def __str__(self) -> str:
        return (
            f'[AnyRun Exception] '
            f'Status: {self._reason.get("status")}. '
            f'Status code: {int(self._reason.get("code")) if self._reason.get("code") else "unspecified"}. '
            f'Description: {self._reason.get("description")}'
        )

    @property
    def json(self) -> dict:
        return self._reason

    @property
    def status(self) -> str:
        return self._reason.get('status')

    @property
    def status_code(self) -> Optional[str]:
        return self._reason.get('code')

    @property
    def description(self) -> str:
        return self._reason.get('description')



from anyrun.iterators.threat_intelligence.feeds.network_iocs_iterator import NetworkIOCsFeedsIterator
from anyrun.iterators.threat_intelligence.feeds.stix_iterator import StixFeedsIterator
from anyrun.iterators.threat_intelligence.feeds.misp_iterator import MispFeedsIterator



from uuid import UUID
from typing import Optional, Iterator, AsyncIterator

import aiohttp
import asyncio

from anyrun.connectors.base_connector import AnyRunConnector
from anyrun.utils.config import Config
from anyrun.utils.utility_functions import execute_synchronously, execute_async_iterator

class YaraLookupConnector(AnyRunConnector):
    """
    Provides ANY.RUN TI Yara Lookup endpoints management.
    Uses aiohttp library for the asynchronous calls
    """
    def __init__(
            self,
            api_key: str,
            user_agent: str = Config.PUBLIC_USER_AGENT,
            trust_env: bool = False,
            verify_ssl: bool = False,
            proxy: Optional[str] = None,
            proxy_auth: Optional[str] = None,
            connector: Optional[aiohttp.BaseConnector] = None,
            timeout: int = Config.DEFAULT_REQUEST_TIMEOUT_IN_SECONDS
    ) -> None:
        """
        :param api_key: ANY.RUN API Key in format: API-KEY <api_key>
        :param user_agent: User-Agent header value
        :param trust_env: Trust environment settings for proxy configuration
        :param verify_ssl: Perform SSL certificate validation for HTTPS requests
        :param proxy: Proxy url
        :param proxy_auth: Proxy authorization url
        :param connector: A custom aiohttp connector
        :param timeout: Override the session’s timeout
        """
        super().__init__(
            api_key,
            user_agent,
            trust_env,
            verify_ssl,
            proxy,
            proxy_auth,
            connector,
            timeout
        )

    def run_yara_search(self, yara_rule: str) -> str:
        """
         Initializes a new search according to the specified YARA rule
`
        :param yara_rule: Valid YARA rule
        :return: Search ID
        """
        return execute_synchronously(self.run_yara_search_async, yara_rule)

    async def run_yara_search_async(self, yara_rule: str) -> UUID:
        """
        Initializes a new search according to the specified YARA rule
`
        :param yara_rule: Valid YARA rule
        :return: Search ID
        """
        url = f'{Config.ANY_RUN_API_URL}/intelligence/yara-lookup/search'
        body = {'query': yara_rule}

        response_data = await self._make_request_async('POST', url, json=body)
        return response_data.get('queryId')

    def get_search_status(self, search_uuid: UUID, simplify: bool = True) -> Iterator[dict]:
        """
        Returns a synchronous iterator to process the actual status until the task is completed.

        :param search_uuid: Search ID
        :param simplify: Returns a simplified dict with the current search status
        :return: Number of matches
        """

        return execute_async_iterator(self.get_search_status_async(search_uuid, simplify))

    async def get_search_status_async(self, search_uuid: UUID, simplify: bool = True) -> AsyncIterator[dict]:
        """
        Returns an asynchronous iterator to process the actual status until the task is completed.

        :param search_uuid: Search ID
        :param simplify: Returns a simplified dict with the current search status
        :return: Number of matches
        """
        url = f'{Config.ANY_RUN_API_URL}/intelligence/yara-lookup/search/{search_uuid}/count'

        while True:
            response_data = await self._make_request_async('GET', url)

            if response_data.get('searchInfo').get('status') == 'done':
                yield await self._prepare_response(response_data, simplify)
                break
            else:
                yield await self._prepare_response(response_data, simplify)

            await asyncio.sleep(Config.DEFAULT_WAITING_TIMEOUT_IN_SECONDS)

    def get_search_result(self, search_uuid: UUID, simplify: bool = False) -> Optional[list[dict]]:
        """
        Returns a list of YARA search matches

        :param search_uuid: Search ID
        :param simplify: Return None if no threats has been detected
        :return: API response in specified format. Returns an empty list if no matches are found
        """
        return execute_synchronously(self.get_search_result_async, search_uuid, simplify)

    async def get_search_result_async(self, search_uuid: UUID, simplify: bool = False) -> Optional[list[dict]]:
        """
        Returns a list of YARA search matches

        :param search_uuid: Search ID
        :param simplify: Return None if no threats has been detected
        :return: API response in specified format. Returns an empty list if no matches are found
        """
        url = f'{Config.ANY_RUN_API_URL}/intelligence/yara-lookup/search/{search_uuid}'

        response_data = await self._make_request_async('GET', url)
        matches = response_data.get('matches')

        if simplify and not matches:
            return
        return matches

    def get_stix_search_result(self, search_uuid: UUID, simplify: bool = False) -> Optional[list[dict]]:
        """
        Returns a list of YARA search matches in stix format

        :param search_uuid: Search ID
        :param simplify: Returns None if no threats has been detected
        :return: API response in specified format. Returns an empty list if no matches are found
        """
        return execute_synchronously(self.get_stix_search_result_async, search_uuid, simplify)

    async def get_stix_search_result_async(self, search_uuid: UUID, simplify: bool = False) -> Optional[list[dict]]:
        """
        Returns a list of YARA search matches in stix format

        :param search_uuid: Search ID
        :param simplify: Returns None if no threats has been detected
        :return: API response in specified format. Returns an empty list if no matches are found
        """
        url = f'{Config.ANY_RUN_API_URL}/intelligence/yara-lookup/search/{search_uuid}/download/stix'

        response_data = await self._make_request_async('GET', url)
        objects = response_data.get('data').get('objects')

        if simplify and not objects:
            return
        return objects

    def get_yara(self, yara_rule: str, stix: bool = False) -> list[Optional[dict]]:
        """
        Automate YARA search methods management. Returns a complete list of YARA search matches

        :param yara_rule: Valid YARA rule
        :param stix: Enable/disable receiving matches in stix format
        :return: API response in specified format. Returns an empty list if no matches are found
        """
        return execute_synchronously(self.get_yara_async, yara_rule, stix)

    async def get_yara_async(self, yara_rule: str, stix: bool = False) -> list[Optional[dict]]:
        """
        Automate YARA search methods management. Returns a complete list of YARA search matches

        :param yara_rule: Valid YARA rule
        :param stix: Enable/disable receiving matches in stix format
        :return: API response in specified format. Returns an empty list if no matches are found
        """
        search_uuid = await self.run_yara_search_async(yara_rule)

        # Wait for the search to complete
        async for _ in self.get_search_status_async(search_uuid):
             pass

        if stix:
            return await self.get_search_result_async(search_uuid)
        return await self.get_stix_search_result_async(search_uuid)

    async def _prepare_response(self, response: dict, simplify: bool) -> dict:
        """
        Simplifies response structure if **simplify** parameter is specified

        :param response: Response dict
        :param simplify: Returns a simplified dict with the remaining scan time and the current task status
        :return: Response dict
        """
        if simplify:
            return {
                'status': await self._resolve_task_status(response.get('searchInfo').get('status'))
            }
        return response

    @staticmethod
    async def _resolve_task_status(status: Optional[str]) -> Optional[str]:
        """ Converts status code string representation """
        if status:
            if status == 'new':
                return 'PREPARING'
            if status == 'processing':
                return 'RUNNING'
            if status == 'done':
                return 'COMPLETED'
            return 'FAILED'



from typing_extensions import override

from anyrun.iterators.base_iterator import BaseIterator
from anyrun.connectors.threat_intelligence.yara_lookup_connector import YaraLookupConnector


class JsonYaraIterator(BaseIterator):
    def __init__(
            self,
            connector: YaraLookupConnector,
            yara_rule: str,
            chunk_size: int = 1
    ) -> None:
        """
        Iterates through the yara search matches. Returns matches in **json** format

        :param connector: Connector instance
        :param yara_rule: Valid YARA rule
        :param chunk_size: The number of feed objects to be retrieved each iteration.
            If greater than one, returns the list of objects
        """
        super().__init__(connector, chunk_size=chunk_size)

        self._yara_rule = yara_rule

    @override
    async def _read_next_chunk(self) -> None:
        """ Overrides parent method using Yara Lookup request """
        if self._pages_counter > 1:
            return

        self._buffer = await self._connector.get_yara_async(self._yara_rule)
        self._pages_counter += 1



import asyncio
from typing import Optional
from typing_extensions import Self, Union

from anyrun.connectors import FeedsConnector, YaraLookupConnector

from anyrun.utils.utility_functions import execute_synchronously


class BaseIterator:
    """ Implements custom iterator protocol """
    def __init__(
            self,
            connector: Union[FeedsConnector, YaraLookupConnector],
            chunk_size: int = 1
    ) -> None:
        """
        Iterates through the feeds objects.

        :param connector: Connector instance
        :param chunk_size: The number of feed objects to be retrieved each iteration.
            If greater than one, returns the list of objects
        """

        self._connector = connector
        self._chunk_size = chunk_size

        self._buffer: list[Optional[dict]] = []
        self._pages_counter = 1

    def __iter__(self) -> Self:
        return self

    def __next__(self) -> Union[list[Optional[dict]], dict]:
        try:
            return execute_synchronously(self.__anext__)
        except StopAsyncIteration as exception:
            raise StopIteration from exception

    def __aiter__(self) -> Self:
        return self

    async def __anext__(self) -> Union[list[Optional[dict]], dict]:
        if len(self._buffer) == 0:
            await self._read_next_chunk()

        if len(self._buffer) == 0:
            raise StopAsyncIteration

        return await self._read_buffer()

    async def _read_next_chunk(self) -> None:
        """
        Executes the next request to the specified ANY.RUN endpoint and stores the result in a buffer.
        Method must be implemented in child iterators according to specific service requests
        """
        pass

    async def _read_buffer(self) -> Union[list[Optional[dict]], dict]:
        """
        Returns the next feeds chunk. Returns a single feed if chunk size is equal one.
        Uses asyncio.Lock() to securely use the list

        :return: A single feed or list of feeds
        """
        async with asyncio.Lock():
            if self._chunk_size == 1:
                return self._buffer.pop(0)

            next_chunk = self._buffer[:self._chunk_size]
            del self._buffer[:self._chunk_size]

            return next_chunk



from typing import Optional
from typing_extensions import override

from anyrun.iterators.base_iterator import BaseIterator
from anyrun.connectors.threat_intelligence.feeds_connector import FeedsConnector


class MispFeedsIterator(BaseIterator):
    def __init__(
            self,
            connector: FeedsConnector,
            chunk_size: int = 1,
            ip: bool = True,
            url: bool = True,
            domain: bool = True,
            show_revoked: bool = False,
            get_new_ioc: bool = False,
            period: Optional[str] = None,
            date_from: Optional[int] = None,
            date_to: Optional[int] = None,
            limit: int = 100
    ) -> None:
        """
        Iterates through the feeds objects.

        :param connector: Connector instance
        :param chunk_size: The number of feed objects to be retrieved each iteration.
            If greater than one, returns the list of objects
        :param ip: Enable or disable the IP type from the feed
        :param url: Enable or disable the URL type from the feed
        :param domain: Enable or disable the Domain type from the feed
        :param show_revoked: Enable or disable receiving revoked feeds in report
        :param get_new_ioc: Receive only updated IOCs since the last request
        :param period: Time period to receive IOCs. Supports: day, week, month
        :param date_from: Beginning of the time period for receiving IOCs in timestamp format
        :param date_to: Ending of the time period for receiving IOCs in timestamp format
        :param limit: Number of tasks on a page. Default, all IOCs are included
        """
        super().__init__(connector, chunk_size=chunk_size)

        self._query_params = {
            'ip': ip,
            'url': url,
            'domain': domain,
            'show_revoked': show_revoked,
            'get_new_ioc': get_new_ioc,
            'period': period,
            'date_from': date_from,
            'date_to': date_to,
            'limit': limit
        }

    @override
    async def _read_next_chunk(self) -> None:
        """ Overrides parent method using TI Feeds requests """
        self._buffer = await self._connector.get_misp_async(
            **self._query_params,
            page=self._pages_counter
        )

        self._pages_counter += 1



from typing import Optional

from anyrun.iterators.threat_intelligence.feeds import StixFeedsIterator, MispFeedsIterator, NetworkIOCsFeedsIterator
from anyrun.connectors import FeedsConnector


class FeedsIterator:
    """ Iterator Factory. Creates a concrete iterator instance according to the method called """
    @staticmethod
    def stix(
            connector: FeedsConnector,
            chunk_size: int = 1,
            ip: bool = True,
            url: bool = True,
            domain: bool = True,
            file: bool = True,
            port: bool = True,
            show_revoked: bool = False,
            get_new_ioc: bool = False,
            period: Optional[str] = None,
            date_from: Optional[int] = None,
            date_to: Optional[int] = None,
            limit: int = 100
    ) -> StixFeedsIterator:
        """
        Iterates through the stix feeds.

        :param connector: Connector instance
        :param chunk_size: The number of feed objects to be retrieved each iteration.
            If greater than one, returns the list of objects
        :param ip: Enable or disable the IP type from the feed
        :param url: Enable or disable the URL type from the feed
        :param domain: Enable or disable the Domain type from the feed
        :param file: Enable or disable the File type from the feed
        :param port: Enable or disable the Port type from the feed
        :param show_revoked: Enable or disable receiving revoked feeds in report
        :param get_new_ioc: Receive only updated IOCs since the last request
        :param period: Time period to receive IOCs. Supports: day, week, month
        :param date_from: Beginning of the time period for receiving IOCs in timestamp format
        :param date_to: Ending of the time period for receiving IOCs in timestamp format
        :param limit: Number of tasks on a page. Default, all IOCs are included
        :return: StixFeedsIterator instance
        """
        return StixFeedsIterator(
            connector=connector,
            chunk_size=chunk_size,
            ip=ip,
            url=url,
            domain=domain,
            file=file,
            port=port,
            show_revoked=show_revoked,
            get_new_ioc=get_new_ioc,
            period=period,
            date_from=date_from,
            date_to=date_to,
            limit=limit
        )

    @staticmethod
    def misp(
            connector: FeedsConnector,
            chunk_size: int = 1,
            ip: bool = True,
            url: bool = True,
            domain: bool = True,
            show_revoked: bool = False,
            get_new_ioc: bool = False,
            period: Optional[str] = None,
            date_from: Optional[int] = None,
            date_to: Optional[int] = None,
            limit: int = 100
    ) -> MispFeedsIterator:
        """
        Iterates through the misp feeds.

        :param connector: Connector instance
        :param chunk_size: The number of feed objects to be retrieved each iteration.
            If greater than one, returns the list of objects
        :param ip: Enable or disable the IP type from the feed
        :param url: Enable or disable the URL type from the feed
        :param domain: Enable or disable the Domain type from the feed
        :param show_revoked: Enable or disable receiving revoked feeds in report
        :param get_new_ioc: Receive only updated IOCs since the last request
        :param period: Time period to receive IOCs. Supports: day, week, month
        :param date_from: Beginning of the time period for receiving IOCs in timestamp format
        :param date_to: Ending of the time period for receiving IOCs in timestamp format
        :param limit: Number of tasks on a page. Default, all IOCs are included
        :return: MispFeedsIterator instance
        """
        return MispFeedsIterator(
            connector=connector,
            chunk_size=chunk_size,
            ip=ip,
            url=url,
            domain=domain,
            show_revoked=show_revoked,
            get_new_ioc=get_new_ioc,
            period=period,
            date_from=date_from,
            date_to=date_to,
            limit=limit
        )

    @staticmethod
    def network_iocs(
            connector: FeedsConnector,
            chunk_size: int = 1,
            ip: bool = True,
            url: bool = True,
            domain: bool = True,
            show_revoked: bool = False,
            get_new_ioc: bool = False,
            period: Optional[str] = None,
            date_from: Optional[int] = None,
            date_to: Optional[int] = None,
            limit: int = 100
    ) -> NetworkIOCsFeedsIterator:
        """
        Iterates through the network_iocs feeds.

        :param connector: Connector instance
        :param chunk_size: The number of feed objects to be retrieved each iteration.
            If greater than one, returns the list of objects
        :param ip: Enable or disable the IP type from the feed
        :param url: Enable or disable the URL type from the feed
        :param domain: Enable or disable the Domain type from the feed
        :param show_revoked: Enable or disable receiving revoked feeds in report
        :param get_new_ioc: Receive only updated IOCs since the last request
        :param period: Time period to receive IOCs. Supports: day, week, month
        :param date_from: Beginning of the time period for receiving IOCs in timestamp format
        :param date_to: Ending of the time period for receiving IOCs in timestamp format
        :param limit: Number of tasks on a page. Default, all IOCs are included
        :return: NetworkIOCsFeedsIterator instance
        """
        return NetworkIOCsFeedsIterator(
            connector=connector,
            chunk_size=chunk_size,
            ip=ip,
            url=url,
            domain=domain,
            show_revoked=show_revoked,
            get_new_ioc=get_new_ioc,
            period=period,
            date_from=date_from,
            date_to=date_to,
            limit=limit
        )



import sys

from anyrun.version import __version__


class Config:
    ANY_RUN_API_URL: str = 'https://api.any.run/v1'

    DEFAULT_REQUEST_TIMEOUT_IN_SECONDS: int = 300
    DEFAULT_WAITING_TIMEOUT_IN_SECONDS: int = 3
    PUBLIC_USER_AGENT: str = f'public/{sys.version.split()[0]}'
    SDK_USER_AGENT: str = f'anyrun-sdk/{__version__}'



import asyncio
from typing import Callable, Any, AsyncIterator


def execute_async_iterator(async_iterator: AsyncIterator[dict]) -> Any:
    """ Provides an async iterator management for the synchronous code """
    event_loop = get_running_loop()

    while True:
        try:
            yield event_loop.run_until_complete(async_iterator.__anext__())
        except StopAsyncIteration:
            break


def execute_synchronously(function: Callable, *args, **kwargs) -> Any:
    """ Creates a coroutine function using the given parameters and then executes it in the event loop """
    event_loop = get_running_loop()
    return event_loop.run_until_complete(function(*args, **kwargs))


def get_running_loop() -> asyncio.AbstractEventLoop:
    """ Tries to get a running event loop. If not found, creates a new one and returns it """
    try:
        return asyncio.get_event_loop()
    except RuntimeError:
        event_loop = asyncio.new_event_loop()
        asyncio.set_event_loop(event_loop)
        return event_loop






from http import HTTPStatus
from typing import Optional, Union
from typing_extensions import Self

import aiohttp
import asyncio

from anyrun.utils.config import Config
from anyrun.utils.exceptions import RunTimeException
from anyrun.utils.utility_functions import execute_synchronously, get_running_loop


class AnyRunConnector:

    def __init__(
            self,
            api_key: str,
            user_agent: str = Config.PUBLIC_USER_AGENT,
            trust_env: bool = False,
            verify_ssl: bool = False,
            proxy: Optional[str] = None,
            proxy_auth: Optional[str] = None,
            connector: Optional[aiohttp.BaseConnector] = None,
            timeout: int = Config.DEFAULT_REQUEST_TIMEOUT_IN_SECONDS,
    ) -> None:
        """
        :param api_key: ANY.RUN Feeds API Key in format: Basic <base64_auth>
        :param user_agent: User-Agent header value
        :param trust_env: Trust environment settings for proxy configuration
        :param verify_ssl: Perform SSL certificate validation for HTTPS requests
        :param proxy: Proxy url
        :param proxy_auth: Proxy authorization url
        :param connector: A custom aiohttp connector
        :param timeout: Override the session’s timeout
        """
        self._proxy = proxy
        self._proxy_auth = proxy_auth
        self._trust_env = trust_env
        self._connector = connector
        self._timeout = timeout
        self._verify_ssl = verify_ssl
        self._session: Optional[aiohttp.ClientSession] = None

        self._api_key_validator(api_key)
        self._setup_connector()
        self._setup_headers(api_key, user_agent)

    def __enter__(self) -> Self:
        execute_synchronously(self._open_session)
        return self

    def __exit__(self, item_type, value, traceback) -> None:
        execute_synchronously(self._close_session)

    async def __aenter__(self) -> Self:
        await self._open_session()
        return self

    async def __aexit__(self, item_type, value, traceback) -> None:
        await self._close_session()

    async def _make_request_async(
            self,
            method: str,
            url: str,
            json: Optional[dict] = None,
            data: Union[dict, aiohttp.MultipartWriter, None] = None,
            parse_response: bool = True
    ) -> Union[dict, aiohttp.ClientResponse]:
        """
        Provides async interface for making any request

        :param method: HTTP method
        :param url: Request url
        :param json: Request json
        :param data: Request data
        :param parse_response: Enable/disable API response parsing. If enabled, returns response.json() object dict
            else aiohttp.ClientResponse instance
        :return: Api response
        :raises RunTimeException: If the connector was executed outside the context manager
        """
        try:
            response: aiohttp.ClientResponse = await self._session.request(
                method,
                url,
                json=json,
                data=data,
                ssl=self._verify_ssl
            )
        except AttributeError:
            raise RunTimeException(
                {
                    'status': 'error',
                    'description': 'The connector object must be executed using the context manager'
                }
            )

        if parse_response:
            response_data = await response.json()
            return await self._check_response_status(response_data, response.status)
        return response

    def _setup_connector(self) -> None:
        if not self._connector and self._verify_ssl:
            event_loop = get_running_loop()
            asyncio.set_event_loop(event_loop)
            self._connector = aiohttp.TCPConnector(ssl=self._verify_ssl, loop=event_loop)

    def _setup_headers(self, api_key: str, user_agent: str) -> None:
        self._headers = {
            'Authorization': api_key,
            'User-Agent': f'{user_agent}:{Config.SDK_USER_AGENT}'
        }

    async def _open_session(self) -> None:
        if not self._session:
            self._session = aiohttp.ClientSession(
                trust_env=self._trust_env,
                connector=self._connector,
                proxy=self._proxy,
                proxy_auth=self._proxy_auth,
                timeout=aiohttp.ClientTimeout(total=self._timeout),
                headers=self._headers
            )

    async def _close_session(self) -> None:
        if self._session:
            await self._session.close()
            self._session = None

    @staticmethod
    async def _check_response_status(response_data: dict, status: int) -> dict:
        """
        Process ANY.RUN endpoint response.

        Returns a dictionary with an explanation of the error if the response status code is not equal **OK**

        :param response_data: API response
        :return: The collection of IOCs
        :raises RunTimeException: If status code 200 is not received
        """
        if status in (HTTPStatus.OK, HTTPStatus.CREATED, HTTPStatus.ACCEPTED):
            return response_data

        raise RunTimeException(
            {
                'status': 'error',
                'code': response_data.get('code') or HTTPStatus.BAD_REQUEST,
                'description': response_data.get('message')
            }
        )

    @staticmethod
    def _api_key_validator(api_key: str) -> None:
        """
        Checks if API key format is valid

        :param api_key:
        :raises RunTimeException: If API key format is not valid
        """
        if not isinstance(api_key, str):
            raise RunTimeException(
                {
                    'status': 'error',
                    'description': 'The ANY.RUN api key must be a valid string'
                }
            )



from typing import Optional, Union, Any

import aiohttp

from anyrun.connectors.base_connector import AnyRunConnector

from anyrun.utils.config import Config
from anyrun.utils.utility_functions import execute_synchronously


class FeedsConnector(AnyRunConnector):
    """
    Provides ANY.RUN TI Feeds endpoints management.
    Uses aiohttp library for the asynchronous calls
    """
    def __init__(
            self,
            api_key: str,
            user_agent: str = Config.PUBLIC_USER_AGENT,
            trust_env: bool = False,
            verify_ssl: bool = False,
            proxy: Optional[str] = None,
            proxy_auth: Optional[str] = None,
            connector: Optional[aiohttp.BaseConnector] = None,
            timeout: int = Config.DEFAULT_REQUEST_TIMEOUT_IN_SECONDS
    ) -> None:
        """
        :param api_key: ANY.RUN API Key in format: Basic <base64_auth>
        :param user_agent: User-Agent header value
        :param trust_env: Trust environment settings for proxy configuration
        :param verify_ssl: Perform SSL certificate validation for HTTPS requests
        :param proxy: Proxy url
        :param proxy_auth: Proxy authorization url
        :param connector: A custom aiohttp connector
        :param timeout: Override the session’s timeout
        """
        super().__init__(
            api_key,
            user_agent,
            trust_env,
            verify_ssl,
            proxy,
            proxy_auth,
            connector,
            timeout
        )

    def get_stix(
            self,
            ip: bool = True,
            url: bool = True,
            domain: bool = True,
            file: bool = True,
            port: bool = True,
            show_revoked: bool = False,
            get_new_ioc: bool = False,
            period: Optional[str] = None,
            date_from: Optional[int] = None,
            date_to: Optional[int] = None,
            limit: int = 100,
            page: int = 1
    ) -> list[Optional[dict]]:
        """
        Returns a list of ANY.RUN Feeds stix objects according to the specified query parameters

        :param ip: Enable or disable the IP type from the feed
        :param url: Enable or disable the URL type from the feed
        :param domain: Enable or disable the Domain type from the feed
        :param file: Enable or disable the File type from the feed
        :param port: Enable or disable the Port type from the feed
        :param show_revoked: Enable or disable receiving revoked feeds in report
        :param get_new_ioc: Receive only updated IOCs since the last request
        :param period: Time period to receive IOCs. Supports: day, week, month
        :param date_from: Beginning of the time period for receiving IOCs in timestamp format
        :param date_to: Ending of the time period for receiving IOCs in timestamp format
        :param limit: Number of tasks on a page. Default, all IOCs are included
        :param page: Page number. The last page marker is a response with a single **identity** object
        :return: The list of feeds in **stix** format
        """
        return execute_synchronously(
            self.get_stix_async,
            ip,
            url,
            domain,
            file,
            port,
            show_revoked,
            get_new_ioc,
            period,
            date_from,
            date_to,
            limit,
            page
        )

    async def get_stix_async(
            self,
            ip: bool = True,
            url: bool = True,
            domain: bool = True,
            file: bool = True,
            port: bool = True,
            show_revoked: bool = False,
            get_new_ioc: bool = False,
            period: Optional[str] = None,
            date_from: Optional[int] = None,
            date_to: Optional[int] = None,
            limit: int = 100,
            page: int = 1
    ) -> list[Optional[dict]]:
        """
        Returns a list of ANY.RUN Feeds stix objects according to the specified query parameters

        :param ip: Enable or disable the IP type from the feed
        :param url: Enable or disable the URL type from the feed
        :param domain: Enable or disable the Domain type from the feed
        :param file: Enable or disable the File type from the feed
        :param port: Enable or disable the Port type from the feed
        :param show_revoked: Enable or disable receiving revoked feeds in report
        :param get_new_ioc: Receive only updated IOCs since the last request
        :param period: Time period to receive IOCs. Supports: day, week, month
        :param date_from: Beginning of the time period for receiving IOCs in timestamp format
        :param date_to: Ending of the time period for receiving IOCs in timestamp format
        :param limit: Number of tasks on a page. Default, all IOCs are included
        :param page: Page number. The last page marker is a response with a single **identity** object
        :return: The list of feeds in **stix** format
        """
        url = await self._generate_feeds_url(
            'stix',
            {
                'IP': ip,
                'URL': url,
                'Domain': domain,
                'File': file,
                'Port': port,
                'showRevoked': show_revoked,
                'GetNewIoc': get_new_ioc,
                'period': period,
                'from': date_from,
                'to': date_to,
                'limit': limit,
                'page': page
             }
        )

        response_data = await self._make_request_async('GET', url)
        return response_data.get('data').get('objects')


    def get_misp(
            self,
            ip: bool = True,
            url: bool = True,
            domain: bool = True,
            show_revoked: bool = False,
            get_new_ioc: bool = False,
            period: Optional[str] = None,
            date_from: Optional[int] = None,
            date_to: Optional[int] = None,
            limit: int = 100,
            page: int = 1
    ) -> list[Optional[dict]]:
        """
        Returns a list of ANY.RUN Feeds misp objects according to the specified query parameters

        :param ip: Enable or disable the IP type from the feed
        :param url: Enable or disable the URL type from the feed
        :param domain: Enable or disable the Domain type from the feed
        :param show_revoked: Enable or disable receiving revoked feeds in report
        :param get_new_ioc: Receive only updated IOCs since the last request
        :param period: Time period to receive IOCs. Supports: day, week, month
        :param date_from: Beginning of the time period for receiving IOCs in timestamp format
        :param date_to: Ending of the time period for receiving IOCs in timestamp format
        :param limit: Number of tasks on a page. Default, all IOCs are included
        :param page: Page number. The last page marker is a response with a single **identity** object
        :return: The list of feeds in **misp** format
        """
        return execute_synchronously(
            self.get_misp_async,
            ip,
            url,
            domain,
            show_revoked,
            get_new_ioc,
            period,
            date_from,
            date_to,
            limit,
            page
        )

    async def get_misp_async(
            self,
            ip: bool = True,
            url: bool = True,
            domain: bool = True,
            show_revoked: bool = False,
            get_new_ioc: bool = False,
            period: Optional[str] = None,
            date_from: Optional[int] = None,
            date_to: Optional[int] = None,
            limit: int = 100,
            page: int = 1
    ) -> list[Optional[dict]]:
        """
        Returns a list of ANY.RUN Feeds misp objects according to the specified query parameters

        :param ip: Enable or disable the IP type from the feed
        :param url: Enable or disable the URL type from the feed
        :param domain: Enable or disable the Domain type from the feed
        :param show_revoked: Enable or disable receiving revoked feeds in report
        :param get_new_ioc: Receive only updated IOCs since the last request
        :param period: Time period to receive IOCs. Supports: day, week, month
        :param date_from: Beginning of the time period for receiving IOCs in timestamp format
        :param date_to: Ending of the time period for receiving IOCs in timestamp format
        :param limit: Number of tasks on a page. Default, all IOCs are included
        :param page: Page number. The last page marker is a response with a single **identity** object
        :return: The list of feeds in **misp** format
        """
        url = await self._generate_feeds_url(
            'misp',
            {
                'IP': ip,
                'URL': url,
                'Domain': domain,
                'showRevoked': show_revoked,
                'GetNewIoc': get_new_ioc,
                'period': period,
                'from': date_from,
                'to': date_to,
                'limit': limit,
                'page': page
            }
        )
        
        response_data = await self._make_request_async('GET', url)
        return response_data.get('data')

    def get_network_iocs(
            self,
            ip: bool = True,
            url: bool = True,
            domain: bool = True,
            show_revoked: bool = False,
            get_new_ioc: bool = False,
            period: Optional[str] = None,
            date_from: Optional[int] = None,
            date_to: Optional[int] = None,
            limit: int = 100,
            page: int = 1
    ) -> list[Optional[dict]]:
        """
        Returns a list of ANY.RUN Feeds network iocs objects according to the specified query parameters

        :param ip: Enable or disable the IP type from the feed
        :param url: Enable or disable the URL type from the feed
        :param domain: Enable or disable the Domain type from the feed
        :param show_revoked: Enable or disable receiving revoked feeds in report
        :param get_new_ioc: Receive only updated IOCs since the last request
        :param period: Time period to receive IOCs. Supports: day, week, month
        :param date_from: Beginning of the time period for receiving IOCs in timestamp format
        :param date_to: Ending of the time period for receiving IOCs in timestamp format
        :param limit: Number of tasks on a page. Default, all IOCs are included
        :param page: Page number. The last page marker is a response with a single **identity** object
        :return: The list of feeds in **network_iocs** format
        """
        return execute_synchronously(
            self.get_network_iocs_async,
            ip,
            url,
            domain,
            show_revoked,
            get_new_ioc,
            period,
            date_from,
            date_to,
            limit,
            page
        )

    async def get_network_iocs_async(
            self,
            ip: bool = True,
            url: bool = True,
            domain: bool = True,
            show_revoked: bool = False,
            get_new_ioc: bool = False,
            period: Optional[str] = None,
            date_from: Optional[int] = None,
            date_to: Optional[int] = None,
            limit: int = 100,
            page: int = 1
    ) -> list[Optional[dict]]:
        """
        Returns a list of ANY.RUN Feeds network iocs objects according to the specified query parameters

        :param ip: Enable or disable the IP type from the feed
        :param url: Enable or disable the URL type from the feed
        :param domain: Enable or disable the Domain type from the feed
        :param show_revoked: Enable or disable receiving revoked feeds in report
        :param get_new_ioc: Receive only updated IOCs since the last request
        :param period: Time period to receive IOCs. Supports: day, week, month
        :param date_from: Beginning of the time period for receiving IOCs in timestamp format
        :param date_to: Ending of the time period for receiving IOCs in timestamp format
        :param limit: Number of tasks on a page. Default, all IOCs are included
        :param page: Page number. The last page marker is a response with a single **identity** object
        :return: The list of feeds in **network_iocs** format
        """
        url = await self._generate_feeds_url(
            'network_iocs',
            {
                'IP': ip,
                'URL': url,
                'Domain': domain,
                'showRevoked': show_revoked,
                'GetNewIoc': get_new_ioc,
                'period': period,
                'from': date_from,
                'to': date_to,
                'limit': limit,
                'page': page
            }
        )

        response_data = await self._make_request_async('GET', url)
        return response_data.get('data')

    async def _generate_feeds_url(self, feed_format: str, params: dict) -> str:
        """
        Builds complete request url according to specified parameters

        :param feed_format: Supports: stix, misp, network_iocs
        :param params: Dictionary with query parameters
        :return: Complete url
        """
        url = f'{Config.ANY_RUN_API_URL}/feeds/{feed_format}.json?'
        query_params = '&'.join(
            [
                f'{param}={await self._parse_boolean(value)}'
                for param, value in params.items() if value
            ]
        )
        return url + query_params

    @staticmethod
    async def _parse_boolean(param: Any) -> Union[str, Any]:
        """ Converts a boolean value to a lowercase string """
        return str(param).lower() if str(param) in ("True", "False") else param



import os
import json
from uuid import UUID
from typing import Optional, Union, BinaryIO, AsyncIterator, Iterator

import aiohttp
import aiofiles

from anyrun.connectors.base_connector import AnyRunConnector
from anyrun.utils.config import Config
from anyrun.utils.exceptions import RunTimeException
from anyrun.utils.utility_functions import execute_synchronously, execute_async_iterator


class SandBoxConnector(AnyRunConnector):
    """
    Provides ANY.RUN TI Yara Lookup endpoints management.
    Uses aiohttp library for the asynchronous calls
    """
    def __init__(
            self,
            api_key: str,
            user_agent: str = Config.PUBLIC_USER_AGENT,
            trust_env: bool = False,
            verify_ssl: bool = False,
            proxy: Optional[str] = None,
            proxy_auth: Optional[str] = None,
            connector: Optional[aiohttp.BaseConnector] = None,
            timeout: int = Config.DEFAULT_REQUEST_TIMEOUT_IN_SECONDS
    ) -> None:
        """
        :param api_key: ANY.RUN API Key in format: API-KEY <api_key> or Basic <base64_auth>
        :param user_agent: User-Agent header value
        :param trust_env: Trust environment settings for proxy configuration
        :param verify_ssl: Perform SSL certificate validation for HTTPS requests
        :param proxy: Proxy url
        :param proxy_auth: Proxy authorization url
        :param connector: A custom aiohttp connector
        :param timeout: Override the session’s timeout
        """
        super().__init__(
            api_key,
            user_agent,
            trust_env,
            verify_ssl,
            proxy,
            proxy_auth,
            connector,
            timeout
        )

    def get_analysis_history(
            self,
            team: bool = False,
            skip: int = 0,
            limit: int = 25
    ) -> list[Optional[dict]]:
        """
        Returns last tasks from the user's history and their basic information

        :param team: Leave this field blank to get your history or specify to get team history
        :param skip: Skip the specified number of tasks
        :param limit: Specify the number of tasks in the result set (not more than 100).
        :return: The list of tasks
        """
        return execute_synchronously(self.get_analysis_history_async, team, skip, limit)

    async def get_analysis_history_async(
            self,
            team: bool = False,
            skip: int = 0,
            limit: int = 25
    ) -> list[Optional[dict]]:
        """
        Returns last tasks from the user's history and their basic information

        :param team: Leave this field blank to get your history or specify to get team history
        :param skip: Skip the specified number of tasks
        :param limit: Specify the number of tasks in the result set (not more than 100).
        :return: The list of tasks
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis'
        body = {
            'team': team,
            'skip': skip,
            'limit': limit
        }

        response_data = await self._make_request_async('GET', url, json=body)
        return response_data.get('data').get('tasks')

    def get_analysis_report(
            self,
            task_uuid: UUID,
            simplify: bool = False
    ) -> Optional[dict]:
        """
        Returns a submission analysis report by task ID

        :param task_uuid: Task uuid
        :param simplify: Return None if no threats found during analysis
        :return: Complete report in **json** format
        """
        return execute_synchronously(self.get_analysis_report_async, task_uuid, simplify)

    async def get_analysis_report_async(
            self,
            task_uuid: UUID,
            simplify: bool = False
    ) -> Optional[dict]:
        """
        Returns a submission analysis report by task ID

        :param task_uuid: Task uuid
        :param simplify: Return None if no threats has been detected
        :return: Complete report in **json** format
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis/{task_uuid}'

        response_data = await self._make_request_async('GET', url)

        if simplify and not await self._find_threats(response_data):
            return
        return response_data.get('data')

    def add_time_to_task(self, task_uuid: UUID) -> dict:
        """
        Adds 60 seconds of execution time to an active task. The task must belong to the current user

        :param task_uuid: Task uuid
        :return: API response json
        """
        return execute_synchronously(self.add_time_to_task_async, task_uuid)

    async def add_time_to_task_async(self, task_uuid: UUID) -> dict:
        """
        Adds 60 seconds of execution time to an active task. The task must belong to the current user

        :param task_uuid: Task uuid
        :return: API response json
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis/addtime/{task_uuid}'
        return await self._make_request_async('PATCH', url)

    def stop_task(self, task_uuid: UUID) -> dict:
        """
        Stops running task. The task must belong to the current user

        :param task_uuid: Task uuid
        :return: API response json
        """
        return execute_synchronously(self.stop_task_async, task_uuid)

    async def stop_task_async(self, task_uuid: UUID) -> dict:
        """
        Stops running task. The task must belong to the current user

        :param task_uuid: Task uuid
        :return: API response json
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis/stop/{task_uuid}'
        return await self._make_request_async('PATCH', url)

    def delete_task(self, task_uuid: UUID) -> dict:
        """
        Deletes running task. The task must belong to the current user

        :param task_uuid: Task uuid
        :return: API response json
        """
        return execute_synchronously(self.delete_task_async, task_uuid)

    async def delete_task_async(self, task_uuid: UUID) -> dict:
        """
        Deletes running task. The task must belong to the current user

        :param task_uuid: Task uuid
        :return: API response json
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis/delete/{task_uuid}'
        return await self._make_request_async('DELETE', url)

    def get_task_status(self, task_uuid: UUID, simplify: bool = True) -> Iterator[dict]:
        """
        Information about the task status is sent to the event stream.
        Returns a synchronous iterator to process the actual status until the task is completed.

        :param task_uuid: Task uuid
        :param simplify: If enabled, returns a simplified dict with the remaining scan time and the current task status
            else returns the entire response
        """
        return execute_async_iterator(self.get_task_status_async(task_uuid, simplify))

    async def get_task_status_async(self, task_uuid: UUID, simplify: bool = True) -> AsyncIterator[dict]:
        """
        Information about the task status is sent to the event stream.
        Returns an asynchronous iterator to process the actual status until the task is completed.

        :param task_uuid: Task uuid
        :param simplify: Returns a simplified dict with the remaining scan time and the current task status
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis/monitor/{task_uuid}'
        response_data = await self._make_request_async('GET', url, parse_response=False)

        await self._check_response_content_type(response_data)

        while True:
            # Read the next chunk from the event stream
            chunk = await response_data.content.readuntil(b'\n')
            # Skip the end of chunk and any meta information
            # https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#fields
            if chunk == b'\n' or any(chunk.startswith(prefix) for prefix in [b"id", b"event", b"entry"]):
                continue
            # Stop interation if event stream is closed
            elif not chunk:
                break
            # Decode and yield the next chunk
            yield await self._prepare_response(chunk, simplify)

    def get_user_environment(self) -> dict:
        """
        Request available user's environment

        :return: API response json
        """
        return execute_synchronously(self.get_user_environment_async)

    async def get_user_environment_async(self) -> dict:
        """
        Request available user's environment

        :return: API response json
        """
        url = f'{Config.ANY_RUN_API_URL}/environment'
        return await self._make_request_async('GET', url)

    def get_user_limits(self) -> dict:
        """
        Request user's API limits

        :return: API response json
        """
        return execute_synchronously(self.get_user_limits_async)

    async def get_user_limits_async(self) -> dict:
        """
        Request available user's environment

        :return: API response json
        """
        url = f'{Config.ANY_RUN_API_URL}/user'
        return await self._make_request_async('GET', url)

    def get_user_presets(self) -> dict:
        """
        Request user's presets

        :return: API response json
        """
        return execute_synchronously(self.get_user_presets_async)

    async def get_user_presets_async(self) -> dict:
        """
        Request user's presets

        :return: API response json
        """
        url = f'{Config.ANY_RUN_API_URL}/user/presets'
        return await self._make_request_async('GET', url)

    def run_file_analysis(
            self,
            file: Union[str, bytes],
            env_os: str = 'windows',
            env_version: str = '10',
            env_bitness: int = 64,
            env_type: str = 'complete',
            env_locale: str = 'en-US',
            opt_network_connect: bool = True,
            opt_network_fakenet: bool = False,
            opt_network_tor: bool = False,
            opt_network_geo: str = 'fastest',
            opt_network_mitm: bool = False,
            opt_network_residential_proxy: bool = False,
            opt_network_residential_proxy_geo: str = 'fastest',
            opt_kernel_heavyevasion: bool = False,
            opt_privacy_type: str = 'bylink',
            opt_timeout: int = 60,
            opt_automated_interactivity: bool = True,
            obj_ext_startfolder: str = 'temp',
            obj_ext_cmd: Optional[str] = None,
            obj_force_elevation: bool = False,
            auto_confirm_uac: bool = True,
            run_as_root: bool = False,
            obj_ext_extension: bool = True,
            task_rerun_uuid: Optional[str] = None
    ) -> UUID:
        """
        Initializes a new file analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_

        :param file: File to analyse. Path to file, or bytes object
        :param env_os: Operation System. Supports: **windows, linux**
        :param env_version: Version of OS. Supports: 7, 10, 11 for **windows** and 22.04.2 for **linux**
        :param env_bitness: Bitness of Operation System. Supports 32, 64 for **windows** and 64 for **linux**
        :param env_type: Environment preset type. Supports clean, office, complete for **windows** and
            office for **linux**
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param obj_ext_startfolder: Start object from. Supports: desktop, home, downloads, appdata, temp, windows,
            root
        :param obj_ext_cmd: Optional command line.
        :param obj_force_elevation: Forces the file to execute with elevated privileges and an elevated token
            (for PE32, PE32+, PE64 files only). Use with the parameter env_os: **windows**
        :param auto_confirm_uac: Auto confirm Windows UAC requests. Use with the parameter env_os: **windows**
        :param run_as_root: Run file with superuser privileges. Use with the parameter env_os: **linux**
        :param obj_ext_extension: Change extension to valid
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        return execute_synchronously(
            self.run_file_analysis_async,
            file=file,
            env_os=env_os,
            env_version=env_version,
            env_bitness=env_bitness,
            env_type=env_type,
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            obj_ext_startfolder=obj_ext_startfolder,
            obj_ext_cmd=obj_ext_cmd,
            obj_force_elevation=obj_force_elevation,
            auto_confirm_uac=auto_confirm_uac,
            run_as_root=run_as_root,
            obj_ext_extension=obj_ext_extension,
            task_rerun_uuid=task_rerun_uuid
        )

    async def run_file_analysis_async(
            self,
            file: Union[str, bytes],
            env_os: str = 'windows',
            env_version: str = '10',
            env_bitness: int = 64,
            env_type: str = 'complete',
            env_locale: str = 'en-US',
            opt_network_connect: bool = True,
            opt_network_fakenet: bool = False,
            opt_network_tor: bool = False,
            opt_network_geo: str = 'fastest',
            opt_network_mitm: bool = False,
            opt_network_residential_proxy: bool = False,
            opt_network_residential_proxy_geo: str = 'fastest',
            opt_kernel_heavyevasion: bool = False,
            opt_privacy_type: str = 'bylink',
            opt_timeout: int = 60,
            opt_automated_interactivity: bool = True,
            obj_ext_startfolder: str = 'temp',
            obj_ext_cmd: Optional[str] = None,
            obj_force_elevation: bool = False,
            auto_confirm_uac: bool = True,
            run_as_root: bool = False,
            obj_ext_extension: bool = True,
            task_rerun_uuid: Optional[str] = None
    ) -> UUID:
        """
        Initializes a new file analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_

        :param file: File to analyse. Path to file, or bytes object
        :param env_os: Operation System. Supports: **windows, linux**
        :param env_version: Version of OS. Supports: 7, 10, 11 for **windows** and 22.04.2 for **linux**
        :param env_bitness: Bitness of Operation System. Supports 32, 64 for **windows** and 64 for **linux**
        :param env_type: Environment preset type. Supports clean, office, complete for **windows** and
            office for **linux**
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param obj_ext_startfolder: Start object from. Supports: desktop, home, downloads, appdata, temp, windows,
            root
        :param obj_ext_cmd: Optional command line.
        :param obj_force_elevation: Forces the file to execute with elevated privileges and an elevated token
            (for PE32, PE32+, PE64 files only). Use with the parameter env_os: **windows**
        :param auto_confirm_uac: Auto confirm Windows UAC requests. Use with the parameter env_os: **windows**
        :param run_as_root: Run file with superuser privileges. Use with the parameter env_os: **linux**
        :param obj_ext_extension: Change extension to valid
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis'

        body = await self._generate_multipart_request_body(
            file,
            env_os=env_os,
            env_version=env_version,
            env_bitness=env_bitness,
            env_type=env_type,
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            obj_ext_startfolder=obj_ext_startfolder,
            obj_ext_cmd=obj_ext_cmd,
            obj_force_elevation=obj_force_elevation,
            auto_confirm_uac=auto_confirm_uac,
            run_as_root=run_as_root,
            obj_ext_extension=obj_ext_extension,
            task_rerun_uuid=task_rerun_uuid,
        )

        response_data = await self._make_request_async('POST', url, data=body)
        return response_data.get('data').get('taskid')

    def run_url_analysis(
            self,
            obj_url: str,
            env_os: str = 'windows',
            env_version: str = '10',
            env_bitness: int = 64,
            env_type: str = 'complete',
            env_locale: str = 'en-US',
            opt_network_connect: bool = True,
            opt_network_fakenet: bool = False,
            opt_network_tor: bool = False,
            opt_network_geo: str = 'fastest',
            opt_network_mitm: bool = False,
            opt_network_residential_proxy: bool = False,
            opt_network_residential_proxy_geo: str = 'fastest',
            opt_kernel_heavyevasion: bool = False,
            opt_privacy_type: str = 'bylink',
            opt_timeout: int = 60,
            opt_automated_interactivity: bool = True,
            obj_ext_browser: str = 'Microsoft Edge',
            obj_ext_extension: bool = True,
            task_rerun_uuid: Optional[str] = None
    ) -> UUID:
        """
        Initializes a new analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_
        
        :param obj_url: Target URL. Size range 5-512. Example: (http/https)://(your-link)
        :param env_os: Operation System. Supports: **windows, linux**
        :param env_version: Version of OS. Supports: 7, 10, 11 for **windows** and 22.04.2 for **linux**
        :param env_bitness: Bitness of Operation System. Supports 32, 64 for **windows** and 64 for **linux**
        :param env_type: Environment preset type. Supports clean, office, complete for **windows** and
            office for **linux**
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param task_rerun_uuid: UUID of the task to be restarted.
        :param obj_ext_browser: Browser name. Supports: Google Chrome
            Mozilla Firefox, Internet Explorer, Microsoft Edge for **windows** and Google Chrome, Mozilla Firefox
            for **linux**
        :param obj_ext_extension: Change extension to valid
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        return execute_synchronously(
            self.run_url_analysis_async,
            obj_url=obj_url,
            env_os=env_os,
            env_version=env_version,
            env_bitness=env_bitness,
            env_type=env_type,
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            task_rerun_uuid=task_rerun_uuid,
            obj_ext_browser=obj_ext_browser,
            obj_ext_extension=obj_ext_extension
        )

    async def run_url_analysis_async(
            self,
            obj_url: str,
            env_os: str = 'windows',
            env_version: str = '10',
            env_bitness: int = 64,
            env_type: str = 'complete',
            env_locale: str = 'en-US',
            opt_network_connect: bool = True,
            opt_network_fakenet: bool = False,
            opt_network_tor: bool = False,
            opt_network_geo: str = 'fastest',
            opt_network_mitm: bool = False,
            opt_network_residential_proxy: bool = False,
            opt_network_residential_proxy_geo: str = 'fastest',
            opt_kernel_heavyevasion: bool = False,
            opt_privacy_type: str = 'bylink',
            opt_timeout: int = 60,
            opt_automated_interactivity: bool = True,
            obj_ext_browser: str = 'Microsoft Edge',
            obj_ext_extension: bool = True,
            task_rerun_uuid: Optional[str] = None
    ) -> UUID:
        """
        Initializes a new analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_
        
        :param obj_url: Target URL. Size range 5-512. Example: (http/https)://(your-link)
        :param env_os: Operation System. Supports: **windows, linux**
        :param env_version: Version of OS. Supports: 7, 10, 11 for **windows** and 22.04.2 for **linux**
        :param env_bitness: Bitness of Operation System. Supports 32, 64 for **windows** and 64 for **linux**
        :param env_type: Environment preset type. Supports clean, office, complete for **windows** and
            office for **linux**
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param task_rerun_uuid: UUID of the task to be restarted.
        :param obj_ext_browser: Browser name. Supports: Google Chrome
            Mozilla Firefox, Internet Explorer, Microsoft Edge for **windows** and Google Chrome, Mozilla Firefox
            for **linux**
        :param obj_ext_extension: Change extension to valid
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis'

        body = await self._generate_request_body(
            'url',
            obj_url=obj_url,
            env_os=env_os,
            env_version=env_version,
            env_bitness=env_bitness,
            env_type=env_type,
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            task_rerun_uuid=task_rerun_uuid,
            obj_ext_browser=obj_ext_browser,
            obj_ext_extension=obj_ext_extension
        )
        response_data = await self._make_request_async('POST', url, json=body)
        return response_data.get('data').get('taskid')

    def run_download_analysis(
            self,
            obj_url: str,
            env_os: str = 'windows',
            env_version: str = '10',
            env_bitness: int = 64,
            env_type: str = 'complete',
            env_locale: str = 'en-US',
            opt_network_connect: bool = True,
            opt_network_fakenet: bool = False,
            opt_network_tor: bool = False,
            opt_network_geo: str = 'fastest',
            opt_network_mitm: bool = False,
            opt_network_residential_proxy: bool = False,
            opt_network_residential_proxy_geo: str = 'fastest',
            opt_kernel_heavyevasion: bool = False,
            opt_privacy_type: str = 'bylink',
            opt_timeout: int = 60,
            opt_automated_interactivity: bool = True,
            obj_ext_startfolder: str = 'temp',
            obj_ext_cmd: Optional[str] = None,
            obj_ext_useragent: Optional[str] = None,
            obj_ext_extension: bool = True,
            opt_privacy_hidesource: bool = False,
            task_rerun_uuid: Optional[str] = None
    ) -> UUID:
        """
        Initializes a new analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_
        
        :param obj_url: Target URL. Size range 5-512. Example: (http/https)://(your-link)
        :param env_os: Operation System. Supports: **windows, linux**
        :param env_version: Version of OS. Supports: 7, 10, 11 for **windows** and 22.04.2 for **linux**
        :param env_bitness: Bitness of Operation System. Supports 32, 64 for **windows** and 64 for **linux**
        :param env_type: Environment preset type. Supports clean, office, complete for **windows** and
            office for **linux**
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param obj_ext_startfolder: Start object from. Supports: desktop, home, downloads, appdata, temp, windows,
            root
        :param task_rerun_uuid: UUID of the task to be restarted.
        :param obj_ext_cmd: Optional command line.
        :param obj_ext_useragent: User-Agent value.
        :param obj_ext_extension: Change extension to valid
        :param opt_privacy_hidesource: Option for hiding of source URL.
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        return execute_synchronously(
            self.run_download_analysis_async,
            obj_url=obj_url,
            env_os=env_os,
            env_version=env_version,
            env_bitness=env_bitness,
            env_type=env_type,
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            obj_ext_startfolder=obj_ext_startfolder,
            task_rerun_uuid=task_rerun_uuid,
            obj_ext_cmd=obj_ext_cmd,
            obj_ext_useragent=obj_ext_useragent,
            obj_ext_extension=obj_ext_extension,
            opt_privacy_hidesource=opt_privacy_hidesource
        )

    async def run_download_analysis_async(
            self,
            obj_url: str,
            env_os: str = 'windows',
            env_version: str = '10',
            env_bitness: int = 64,
            env_type: str = 'complete',
            env_locale: str = 'en-US',
            opt_network_connect: bool = True,
            opt_network_fakenet: bool = False,
            opt_network_tor: bool = False,
            opt_network_geo: str = 'fastest',
            opt_network_mitm: bool = False,
            opt_network_residential_proxy: bool = False,
            opt_network_residential_proxy_geo: str = 'fastest',
            opt_kernel_heavyevasion: bool = False,
            opt_privacy_type: str = 'bylink',
            opt_timeout: int = 60,
            opt_automated_interactivity: bool = True,
            obj_ext_startfolder: str = 'temp',
            obj_ext_cmd: Optional[str] = None,
            obj_ext_useragent: Optional[str] = None,
            obj_ext_extension: bool = True,
            opt_privacy_hidesource: bool = False,
            task_rerun_uuid: Optional[str] = None
    ) -> UUID:
        """
        Initializes a new analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_
        
        :param obj_url: Target URL. Size range 5-512. Example: (http/https)://(your-link)
        :param env_os: Operation System. Supports: **windows, linux**
        :param env_version: Version of OS. Supports: 7, 10, 11 for **windows** and 22.04.2 for **linux**
        :param env_bitness: Bitness of Operation System. Supports 32, 64 for **windows** and 64 for **linux**
        :param env_type: Environment preset type. Supports clean, office, complete for **windows** and
            office for **linux**
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param obj_ext_startfolder: Start object from. Supports: desktop, home, downloads, appdata, temp, windows,
            root
        :param task_rerun_uuid: UUID of the task to be restarted.
        :param obj_ext_cmd: Optional command line.
        :param obj_ext_useragent: User-Agent value.
        :param obj_ext_extension: Change extension to valid
        :param opt_privacy_hidesource: Option for hiding of source URL.
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis'

        body = await self._generate_request_body(
            'download',
            obj_url=obj_url,
            env_os=env_os,
            env_version=env_version,
            env_bitness=env_bitness,
            env_type=env_type,
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            obj_ext_startfolder=obj_ext_startfolder,
            task_rerun_uuid=task_rerun_uuid,
            obj_ext_cmd=obj_ext_cmd,
            obj_ext_useragent=obj_ext_useragent,
            obj_ext_extension=obj_ext_extension,
            opt_privacy_hidesource=opt_privacy_hidesource
        )

        response_data = await self._make_request_async('POST', url, json=body)
        return response_data.get('data').get('taskid')

    async def _generate_multipart_request_body(
            self,
            file: Union[str, BinaryIO],
            **params,
    ) -> aiohttp.MultipartWriter:
        """
        Generates request body for the **form-data** content type

        :param file: Path to file or bytes
        :param params: Dictionary with task settings
        :return: Request payload stored in aiohttp MultipartWriter object instance
        """
        form_data = aiohttp.MultipartWriter("form-data")

        # Prepare file payload
        file_data = await self._get_file_payload(file)
        filename = f'{os.path.basename(file) if isinstance(file, str) else "sdk_file_analysis"}'
        disposition = f'form-data; name="file"; filename="{filename}"'
        file_data.headers["Content-Disposition"] = disposition
        form_data.append_payload(file_data)

        # Choose a task type
        params = await self._set_task_object_type(params, 'file')

        # Prepare analysis settings payload
        for param, value in params.items():
            if value:
                part = form_data.append(str(value))
                part.set_content_disposition('form-data', name=param)

        return form_data

    async def _generate_request_body(
            self,
            object_type: str,
            **params,
        ) -> dict[str, Union[int, str, bool]]:
        """
         Generates request body for the **application/json** content type

        :param object_type: Sandbox object type
        :param params: Dictionary with task settings
        :return: Request payload stored in dictionary
        """
        request_body = {param: value for param, value in params.items() if value}
        return await self._set_task_object_type(request_body, object_type)

    async def _prepare_response(self, chunk: bytes, simplify: bool) -> dict:
        """
        Deserialize response bytes to dictionary

        :param chunk: Current content chunk
        :param simplify: Returns a simplified dict with the remaining scan time and the current task status
        :return: API response json
        """
        # Exclude 'data: ' field from the chunk and decode entire dictionary
        status_data = json.loads(chunk[6:].decode())

        if simplify:
            return {
                'status': await self._resolve_task_status(status_data.get('task').get('status')),
                'seconds_remaining': status_data.get('task').get('remaining')
            }
        return status_data

    @staticmethod
    async def _find_threats(response_data: dict) -> Optional[bool]:
        """
        Checks whether threats are detected during analysis

        :param response_data: Report in **json** format
        :return: **True** if threats were detected else **None**
        """
        if (
            response_data.get('data')
            .get('analysis')
            .get('scores')
            .get('verdict')
            .get('threatLevelText') != 'No threats detected'
           ):
            return True

    @staticmethod
    async def _check_response_content_type(response: aiohttp.ClientResponse) -> None:
        """
        Checks if the response has **text/event-stream** content-type

        :param response: API response
        :raises RunTimeException: If response has a different content-type
        """
        if not response.content_type.startswith('text/event-stream'):
            raise RunTimeException(
                {
                    'status': 'error',
                    'code': response.status,
                    'description': (await response.json()).get('message')
                }
            )

    @staticmethod
    async def _resolve_task_status(status_code: int) -> str:
        """ Converts an integer status code value to a string representation """
        if status_code == -1:
            return 'FAILED'
        elif 50 <= status_code <= 99:
            return 'RUNNING'
        elif status_code == 100:
            return 'COMPLETED'
        return 'PREPARING'

    @staticmethod
    async def _get_file_payload(file: Union[str, bytes]) -> aiohttp.Payload:
        """
        Generates file payload from received file content. Tries to open a file if given a file path

        :param file: Path to file or bytes
        :return: Aiohttp Payload object instance
        :raises RunTimeException: If invalid filepath is received
        """
        if isinstance(file, bytes):
            return aiohttp.get_payload(file)

        if not os.path.isfile(file):
            raise RunTimeException(
                {
                    'status': 'error',
                    'description': f'Received not valid filepath: {file}'
                }
            )

        async with aiofiles.open(file, mode='rb') as file:
            return aiohttp.get_payload(await file.read())

    @staticmethod
    async def _set_task_object_type(
            params: dict[str, Union[int, str, bool]],
            obj_type: str
    ) -> dict[str, Union[int, str, bool]]:
        """
        Sets **obj_type** value to 'rerun' if **task_rerun_uuid** parameter is not None.
        Otherwise, sets received object type

        :param params: Dictionary with task settings
        :param obj_type: Sandbox task object type
        :return: Dictionary with task settings
        """
        if params.get('task_rerun_uuid'):
            params['obj_type'] = 'rerun'
        else:
            params['obj_type'] = obj_type
        return params



from typing import Optional

import aiohttp

from anyrun.connectors.base_connector import AnyRunConnector

from anyrun.utils.config import Config
from anyrun.utils.utility_functions import execute_synchronously


class LookupConnector(AnyRunConnector):
    """
    Provides ANY.RUN TI Lookup endpoint management.
    Uses aiohttp library for the asynchronous calls
    """
    def __init__(
            self,
            api_key: str,
            user_agent: str = Config.PUBLIC_USER_AGENT,
            trust_env: bool = False,
            verify_ssl: bool = False,
            proxy: Optional[str] = None,
            proxy_auth: Optional[str] = None,
            connector: Optional[aiohttp.BaseConnector] = None,
            timeout: int = Config.DEFAULT_REQUEST_TIMEOUT_IN_SECONDS
    ) -> None:
        """
        :param api_key: ANY.RUN API Key in format: API-KEY <api_key>
        :param user_agent: User-Agent header value
        :param trust_env: Trust environment settings for proxy configuration
        :param verify_ssl: Perform SSL certificate validation for HTTPS requests
        :param proxy: Proxy url
        :param proxy_auth: Proxy authorization url
        :param connector: A custom aiohttp connector
        :param timeout: Override the session’s timeout
        """
        super().__init__(
            api_key,
            user_agent,
            trust_env,
            verify_ssl,
            proxy,
            proxy_auth,
            connector,
            timeout
        )

    def get_intelligence(
            self,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            query: Optional[str] = None,
            threat_name: Optional[str] = None,
            threat_level: Optional[str] = None,
            task_type: Optional[str] = None,
            submission_country: Optional[str] = None,
            os: Optional[str] = None,
            os_software_set: Optional[str] = None,
            os_bit_version: Optional[str] = None,
            registry_key: Optional[str] = None,
            registry_name: Optional[str] = None,
            registry_value: Optional[str] = None,
            module_image_path: Optional[str] = None,
            rule_threat_level: Optional[str] = None,
            rule_name: Optional[str] = None,
            mitre: Optional[str] = None,
            image_path: Optional[str] = None,
            command_line: Optional[str] = None,
            injected_flag: Optional[str] = None,
            destination_ip: Optional[str] = None,
            destination_port: Optional[str] = None,
            destination_ip_asn: Optional[str] = None,
            destination_ip_geo: Optional[str] = None,
            domain_name: Optional[str] = None,
            ja3: Optional[str] = None,
            ja3s: Optional[str] = None,
            jarm : Optional[str] = None,
            file_path: Optional[str] = None,
            file_event_path: Optional[str] = None,
            file_extension: Optional[str] = None,
            sha256: Optional[str] = None,
            sha1: Optional[str] = None,
            md5: Optional[str] = None,
            suricata_class: Optional[str] = None,
            suricata_message: Optional[str] = None,
            suricata_threat_level: Optional[str] = None,
            suricata_id: Optional[str] = None,
            sync_object_name: Optional[str] = None,
            sync_object_type: Optional[str] = None,
            sync_object_operation: Optional[str] = None,
            url: Optional[str] = None,
            http_request_content_type: Optional[str] = None,
            http_response_content_type: Optional[str] = None,
            http_request_file_type: Optional[str] = None,
            http_response_file_type: Optional[str] = None
    ) -> dict:
        """
        Returns Lookup object according to the specified query parameters.
        Supports two ways to build a request query:

        * According to specified keyword arguments. All conditions are combined using the AND operator.
        * According to specified raw query. All conditions are combined using specified operators.

        You cannot use both methods at the same time. If a query and some other filter parameters are specified,
        the query parameter will have higher priority

        :param start_date: Indicating the beginning of the period for which events is requested. Format: YYYY-MM-DD
        :param end_date: Indicating the end of the period for which events is requested. Format: YYYY-MM-DD
        :param query: Raw query with necessary filters. Supports condition concatenation with AND, OR, NOT and
            Parentheses ()
        :param threat_name: The name of a particular threat: malware family, threat type, etc., as identified by the
            sandbox. Example: "Phishing"
        :param threat_level: A verdict on the threat level of the sample. Supports: **suspicious, malicious, info**
        :param task_type: The type of the sample submitted to the sandbox. Supports: **File, URL**
        :param submission_country: The country from which the threat sample was submitted. Example: "es"
        :param os: The specific version of Windows used in the environment. Supports: **Windows 7, Windows 10,
            Windows 11**
        :param os_software_set: The software package of applications installed on the OS. Supports: **clean, office,
            complete**
        :param os_bit_version: The bitness of the operating system. Supports: **32, 64**
        :param registry_key: The specific key within the registry hive where the modification occurred. Please note:
            when entering registry keys, use a double backslash (\) to escape the single backslash. Example:
            "Windows\\CurrentVersion\\RunOnce"
        :param registry_name: The name of the Windows Registry key field. Example: "browseinplace"
        :param registry_value: The value of the Windows Registry key. Example: "Internet Explorer\iexplore.exe"
        :param module_image_path: The full path to the module’s image file, the location on the disk where the module’s
            executable is stored. Example: "SysWOW64\\cryptbase.dll"
        :param rule_threat_level: The threat level assigned to a particular event. Supports: **suspicious, malicious,
            info**
        :param rule_name: The name of the detection rule. Example: "Executable content was dropped or overwritten"
        :param mitre: Techniques used by the malware according to the MITRE ATT&CK classification. Example: "T1071"
        :param image_path: Full path to process image. Example: "System32\\conhost.exe"
        :param command_line: Full command line that initiated the process. Example:
            "PDQConnectAgent\\pdq-connect-agent.exe –service"
        :param injected_flag: Indication of whether a process has been injected. Supports: **"true", "false"**
        :param destination_ip: The IP address of the network connection that was established or attempted. Example:
            "147.185.221.22"
        :param destination_port: The network port through which the connection was established. Example: "49760"
        :param destination_ip_asn: Detected ASN. Example: "akamai-as"
        :param destination_ip_geo: Two-letter country or region code of the detected IP geolocation. Example: "ae"
        :param domain_name: The domain name that was recorded during the threat execution in a sandbox. Example:
            "tventyvd20sb.top"
        :param ja3: Types of TLS fingerprints that can indicate certain threats.
        :param ja3s: Types of TLS fingerprints that can indicate certain threats.
        :param jarm: Types of TLS fingerprints that can indicate certain threats.
        :param file_path: The full path to the file on the system.
        :param file_event_path: The path of a file associated with a file event.
        :param file_extension: The extension that indicates the file type.
        :param sha256: Hash values relating to a file.
        :param sha1: Hash values relating to a file.
        :param md5: Hash values relating to a file.
        :param suricata_class: The category assigned to the threat by Suricata based on its characteristics. Example:
            "a network trojan was detected"
        :param suricata_message: The description of the threat according to Suricata. Example:
            "ET INFO 404/Snake/Matiex Keylogger Style External IP Check"
        :param suricata_threat_level: The verdict on the threat according to Suricata based on its potential impact.
            Supports: **suspicious, malicious, info**
        :param suricata_id: The unique identifier of the Suricata rule: Example: "2044767"
        :param sync_object_name: The name or identifier of the synchronization object used. Example: "rmc"
        :param sync_object_type: The type of synchronization object used. Example: "mutex"
        :param sync_object_operation: The operation performed on the synchronization object. Example: "create"
        :param url: The URL called by the process. Example: "http://192.168.37.128:8880/zv8u"
        :param http_request_content_type: The content type of the HTTP request sent to the server. Example:
            "application/json"
        :param http_response_content_type: The content type of the HTTP response received from the server. Example:
            "text/html"
        :param http_request_file_type: The file type of the file being uploaded in the HTTP request. Example:
            "binary"
        :param http_response_file_type: The file type of the file being downloaded in the HTTP response. Example:
            "binary"
        :return: API response in **json** format
        """
        return execute_synchronously(
            self.get_intelligence_async,
            start_date=start_date,
            end_date=end_date,
            query=query,
            threat_name=threat_name,
            threat_level=threat_level,
            task_type=task_type,
            submission_country=submission_country,
            os=os,
            os_software_set=os_software_set,
            os_bit_version=os_bit_version,
            registry_key=registry_key,
            registry_name=registry_name,
            registry_value=registry_value,
            module_image_path=module_image_path,
            rule_threat_level=rule_threat_level,
            rule_name=rule_name,
            mitre=mitre,
            image_path=image_path,
            command_line=command_line,
            injected_flag=injected_flag,
            destination_ip=destination_ip,
            destination_port=destination_port,
            destination_ip_asn=destination_ip_asn,
            destination_ip_geo=destination_ip_geo,
            domain_name=domain_name,
            ja3=ja3,
            ja3s=ja3s,
            jarm=jarm,
            file_path=file_path,
            file_event_path=file_event_path,
            file_extension=file_extension,
            sha256=sha256,
            sha1=sha1,
            md5=md5,
            suricata_class=suricata_class,
            suricata_message=suricata_message,
            suricata_threat_level=suricata_threat_level,
            suricata_id=suricata_id,
            sync_object_name=sync_object_name,
            sync_object_type=sync_object_type,
            sync_object_operation=sync_object_operation,
            url=url,
            http_request_content_type=http_request_content_type,
            http_response_content_type=http_response_content_type,
            http_request_file_type=http_request_file_type,
            http_response_file_type=http_response_file_type
        )

    async def get_intelligence_async(
            self,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            query: Optional[str] = None,
            threat_name: Optional[str] = None,
            threat_level: Optional[str] = None,
            task_type: Optional[str] = None,
            submission_country: Optional[str] = None,
            os: Optional[str] = None,
            os_software_set: Optional[str] = None,
            os_bit_version: Optional[str] = None,
            registry_key: Optional[str] = None,
            registry_name: Optional[str] = None,
            registry_value: Optional[str] = None,
            module_image_path: Optional[str] = None,
            rule_threat_level: Optional[str] = None,
            rule_name: Optional[str] = None,
            mitre: Optional[str] = None,
            image_path: Optional[str] = None,
            command_line: Optional[str] = None,
            injected_flag: Optional[str] = None,
            destination_ip: Optional[str] = None,
            destination_port: Optional[str] = None,
            destination_ip_asn: Optional[str] = None,
            destination_ip_geo: Optional[str] = None,
            domain_name: Optional[str] = None,
            ja3: Optional[str] = None,
            ja3s: Optional[str] = None,
            jarm : Optional[str] = None,
            file_path: Optional[str] = None,
            file_event_path: Optional[str] = None,
            file_extension: Optional[str] = None,
            sha256: Optional[str] = None,
            sha1: Optional[str] = None,
            md5: Optional[str] = None,
            suricata_class: Optional[str] = None,
            suricata_message: Optional[str] = None,
            suricata_threat_level: Optional[str] = None,
            suricata_id: Optional[str] = None,
            sync_object_name: Optional[str] = None,
            sync_object_type: Optional[str] = None,
            sync_object_operation: Optional[str] = None,
            url: Optional[str] = None,
            http_request_content_type: Optional[str] = None,
            http_response_content_type: Optional[str] = None,
            http_request_file_type: Optional[str] = None,
            http_response_file_type: Optional[str] = None
    ) -> dict:
        """
        Returns Lookup object according to the specified query parameters.
        Supports two ways to build a request query:

        * According to specified keyword arguments. All conditions are combined using the AND operator.
        * According to specified raw query. All conditions are combined using specified operators.

        You cannot use both methods at the same time. If a query and some other filter parameters are specified,
        the query parameter will have higher priority

        :param start_date: Indicating the beginning of the period for which events is requested. Format: YYYY-MM-DD
        :param end_date: Indicating the end of the period for which events is requested. Format: YYYY-MM-DD
        :param query: Raw query with necessary filters. Supports condition concatenation with AND, OR, NOT and
            Parentheses ()
        :param threat_name: The name of a particular threat: malware family, threat type, etc., as identified by the
            sandbox. Example: "Phishing"
        :param threat_level: A verdict on the threat level of the sample. Supports: **suspicious, malicious, info**
        :param task_type: The type of the sample submitted to the sandbox. Supports: **File, URL**
        :param submission_country: The country from which the threat sample was submitted. Example: "es"
        :param os: The specific version of Windows used in the environment. Supports: **Windows 7, Windows 10,
            Windows 11**
        :param os_software_set: The software package of applications installed on the OS. Supports: **clean, office,
            complete**
        :param os_bit_version: The bitness of the operating system. Supports: **32, 64**
        :param registry_key: The specific key within the registry hive where the modification occurred. Please note:
            when entering registry keys, use a double backslash (\) to escape the single backslash. Example:
            "Windows\\CurrentVersion\\RunOnce"
        :param registry_name: The name of the Windows Registry key field. Example: "browseinplace"
        :param registry_value: The value of the Windows Registry key. Example: "Internet Explorer\iexplore.exe"
        :param module_image_path: The full path to the module’s image file, the location on the disk where the module’s
            executable is stored. Example: "SysWOW64\\cryptbase.dll"
        :param rule_threat_level: The threat level assigned to a particular event. Supports: **suspicious, malicious,
            info**
        :param rule_name: The name of the detection rule. Example: "Executable content was dropped or overwritten"
        :param mitre: Techniques used by the malware according to the MITRE ATT&CK classification. Example: "T1071"
        :param image_path: Full path to process image. Example: "System32\\conhost.exe"
        :param command_line: Full command line that initiated the process. Example:
            "PDQConnectAgent\\pdq-connect-agent.exe –service"
        :param injected_flag: Indication of whether a process has been injected. Supports: **"true", "false"**
        :param destination_ip: The IP address of the network connection that was established or attempted. Example:
            "147.185.221.22"
        :param destination_port: The network port through which the connection was established. Example: "49760"
        :param destination_ip_asn: Detected ASN. Example: "akamai-as"
        :param destination_ip_geo: Two-letter country or region code of the detected IP geolocation. Example: "ae"
        :param domain_name: The domain name that was recorded during the threat execution in a sandbox. Example:
            "tventyvd20sb.top"
        :param ja3: Types of TLS fingerprints that can indicate certain threats.
        :param ja3s: Types of TLS fingerprints that can indicate certain threats.
        :param jarm: Types of TLS fingerprints that can indicate certain threats.
        :param file_path: The full path to the file on the system.
        :param file_event_path: The path of a file associated with a file event.
        :param file_extension: The extension that indicates the file type.
        :param sha256: Hash values relating to a file.
        :param sha1: Hash values relating to a file.
        :param md5: Hash values relating to a file.
        :param suricata_class: The category assigned to the threat by Suricata based on its characteristics. Example:
            "a network trojan was detected"
        :param suricata_message: The description of the threat according to Suricata. Example:
            "ET INFO 404/Snake/Matiex Keylogger Style External IP Check"
        :param suricata_threat_level: The verdict on the threat according to Suricata based on its potential impact.
            Supports: **suspicious, malicious, info**
        :param suricata_id: The unique identifier of the Suricata rule: Example: "2044767"
        :param sync_object_name: The name or identifier of the synchronization object used. Example: "rmc"
        :param sync_object_type: The type of synchronization object used. Example: "mutex"
        :param sync_object_operation: The operation performed on the synchronization object. Example: "create"
        :param url: The URL called by the process. Example: (http/https)://(your-link)
        :param http_request_content_type: The content type of the HTTP request sent to the server. Example:
            "application/json"
        :param http_response_content_type: The content type of the HTTP response received from the server. Example:
            "text/html"
        :param http_request_file_type: The file type of the file being uploaded in the HTTP request. Example:
            "binary"
        :param http_response_file_type: The file type of the file being downloaded in the HTTP response. Example:
            "binary"
        :return: API response in **json** format
        """
        body = await self._generate_request_body(
            start_date,
            end_date,
            query,
            {
                'threatName': threat_name,
                'submissionCountry': submission_country,
                'taskType': task_type,
                'threatLevel': threat_level,
                'registryKey': registry_key,
                'registryName': registry_name,
                'registryValue': registry_value,
                'os': os,
                'osSoftwareSet': os_software_set,
                'osBitVersion': os_bit_version,
                'ruleName': rule_name,
                'ruleThreatLevel': rule_threat_level,
                'MITRE': mitre,
                'moduleImagePath': module_image_path,
                'domainName': domain_name,
                'destinationPort': destination_port,
                'destinationIP': destination_ip,
                'destinationIpAsn': destination_ip_asn,
                'destinationIPgeo': destination_ip_geo,
                'ja3': ja3,
                'ja3s': ja3s,
                'jarm': jarm,
                'imagePath': image_path,
                'commandLine': command_line,
                'injectedFlag': injected_flag,
                'suricataMessage': suricata_message,
                'suricataClass': suricata_class,
                'suricataThreatLevel': suricata_threat_level,
                'suricataID': suricata_id,
                'filePath': file_path,
                'fileEventPath': file_event_path,
                'fileExtension': file_extension,
                'sha256': sha256,
                'sha1': sha1,
                'md5': md5,
                'syncObjectName': sync_object_name,
                'syncObjectType': sync_object_type,
                'syncObjectOperation': sync_object_operation,
                'URL': url,
                'httpRequestContentType': http_request_content_type,
                'httpResponseContentType': http_response_content_type,
                'httpRequestFileType': http_request_file_type,
                'httpResponseFileType': http_response_file_type
            }
        )
        
        lookup_url = f'{Config.ANY_RUN_API_URL}/intelligence/search'
        response_data = await self._make_request_async('POST', lookup_url, json=body)
        return response_data

    async def _generate_request_body(
            self,
            start_date: Optional[str],
            end_date: Optional[str],
            query: Optional[str],
            params: dict[str, str]
    ) -> dict[str, str]:
        """
        Builds complete request body according to specified parameters

        :param start_date: Indicating the beginning of the period for which events is requested.
        :param end_date: Indicating the end of the period for which events is requested.
        :param query: Raw query with necessary filters.
        :param params: Dictionary with filter parameters.
        :return:
        """
        body = {'query': query if query else await self._generate_query(params)}
        body = await self._add_time_ranges(body, start_date, end_date)
        return body

    @staticmethod
    async def _generate_query(params: dict[str, str]) -> str:
        """
        Generates filter query using specified parameters

        :param params: Dictionary with filter parameters.
        :return: Complete query
        """
        return ' AND '.join(f'{param}:"{value}"' for param, value in params.items() if value)

    @staticmethod
    async def _add_time_ranges(
            body: dict[str, str],
            start_date: Optional[str],
            end_date: Optional[str]
    ) -> dict[str, str]:
        """
        Checks if time range parameters specified. If specified, appends them to request body

        :param body: Request body
        :param start_date: Indicating the beginning of the period for which events is requested.
        :param end_date: Indicating the end of the period for which events is requested.
        :return: Updated request body
        """
        if start_date:
            body['startDate'] = start_date

        if end_date:
            body['endDate'] = end_date

        return body
